#!/usr/bin/perl

=head1 NAME

weps2sval2.pl

=head1 SYNOPSIS

./wpes2sval2.pl --input <INPUT-DIR> --output <OUTPUT-DIR>

OR 

./weps2sval2.pl --input <INPUT-DIR> --output <OUTPUT-DIR> --include_discards

OR

./wpes2sval2.pl --input <INPUT-DIR> --output <OUTPUT-DIR> --verbose --include_discards

OR

./wpes2sval2.pl --input <INPUT-DIR> --output <OUTPUT-DIR> --verbose

OR

./wpes2sval2.pl --input <INPUT-DIR> --output <OUTPUT-DIR> --raw

OR

./wpes2sval2.pl --input <INPUT-DIR> --output <OUTPUT-DIR> --raw --include_discards

OR

./wpes2sval2.pl --input <INPUT-DIR> --output <OUTPUT-DIR> --raw --include_discards --verbose

=head1 DESCRIPTION

This program converts Web People Search (WePS) Data  in to SensEval-2 format.
WePS data was released as part of the WePS shared task as part of SemEval-2007.
The data consists of web search results for individuals. For more information 
on the task please refer to the SEE ALSO section of this perldoc. 
The WePS data an be obtained from 

 L<http://bender.lsi.uned.es/~javart/weps2007_data_1.0.tar.gz> 

weps2sval2.pl expects 2 mandatory parameters, --input and --output. 

WePS data found at the above link follows the following folder structure. 
weps2sval2.pl assumes some part of this folder structure for its working. After 
unpacking the tar ball archive the following folder structure can be found.

The root folder 'weps2007_data_1.0' contains 3 sub folders "scorer", "test", 
"traininig". The folder scorer has a scoring program provided with the corpus. 
We do not use this folder. Folders "test" and "traininig" are of our interest. 
They contain the search results for the ambiguous name. The "traininig" 
folder has 49 instances of ambiguous names and "test" folder has 30 instances 
of ambiguous names. The data collected per ambiguous name search result is rank 
of this result in all results returned, title of the page, snippet text from 
the web page, and the URL of the web page. Each of these folders "traininig" 
and "test" has a similar sub-folder structure in them. They contain 
"description_files", "truth_files", "web_pages" sub folders. The 
"description_files" folder has XML files. Every XML file describes the raw data 
always with its rank, and URL, and occasionally with title and snippet 
information. The "truth_files" folder has the XMLs with manual annotation of 
clusters of the search results. We read XML files from each of these folders 
and use it to extract rank, title, snippet and answer information per 
ambiguous name. The "web_pages" folder has the raw web pages downloaded and 
stored off-line form the search results. This folder stores all the web pages in 
the raw sub folder with their rank as the parent folder and the html file as 
index.html. We use this index.html file for extracting the raw text for our 
analysis. The "test" folder is similar to "traininig" except that its 
truth_files folder has 3 sub folders that annotation. We use the XMLs 
"official_annotation" folder for extracting answers for the ambiguous name 
instances in the test folder.

Below is the tree representation of the described folder structure.

=over 2

=item + weps2007_data_1.0/

=over 4

=item + scorer/

=item + test/ 

=over 4

=item + description_files/

=over 4

=item Alvin_Cooper.xml

=item Arthur_Morgan.xml

=item ...

=item William_Dickson.xml

=back

=item + truth_files/

=over 4

=item + annotation_2/ 

=item + annotation_1/ 

=item + official_annotation/

=over 4

=item Alvin_Cooper.xml

=item ...

=item William_Dickson.xml

=back 

=back 

=item + web_pages/

=over 4

=item + Alvin_Cooper/

=over 4

=item Alvin_Cooper.xml

=item + raw/ 

=over 4

=item 000/

=over 4

=item index.html

=item *.jpeg

=item *.gif

=back

=item 001/

=over 4

=item index.html

=back

=item ...

=item 099/

=over 4

=item index.html

=back

=back

=back 

=item ...

=item + William_Dickson/

=over 4

=item William_Dickson.xml

=item + raw/ 

=over 4

=item 000/

=over 4

=item index.html

=back

=item 001/

=over 4

=item index.html

=back

=item ...

=item 099/

=over 4

=item index.html

=back

=back 

=back

=item + traininig/ 

=over 4

=item + description_files/

=over 4

=item Alvin_Cooper.xml

=item Arthur_Morgan.xml

=item ...

=item William_Dickson.xml

=back

=item + truth_files/

=over 4

=item Alvin_Cooper.xml

=item ...

=item William_Dickson.xml

=back 

=item + web_pages/

=over 4

=item + Alvin_Cooper/

=over 4

=item Alvin_Cooper.xml

=item + raw/ 

=over 4

=item 000/

=over 4

=item index.html

=back

=item 001/

=over 4

=item index.html

=back

=item ...

=item 099/

=over 4

=item index.html

=back

=back

=back 

=item ...

=item + William_Dickson/

=over 4

=item William_Dickson.xml

=item + raw/ 

=over 4

=item 000/

=over 4

=item index.html

=back

=item 001/

=over 4

=item index.html

=back

=item ...

=item 099/

=over 4

=item index.html

=back

=back 

=back

=back

=back

=back

=back

=back

=back

Please refer to the websites listed in the SEE ALSO Section for for more 
details on the WePS task and "WePS data". 

weps2sval2.pl converts the data either from the snippets (title or snippet 
text) or using the web pages from the WePS corpus to the headless SenseEval-2 
format. The program expects two required parameters. These parameters are 

=head2 --input 

This option takes as its parameter the directory in which the WePS data 
resides. This directory should have structure like the test and traininig 
directories as described above. It is an mandatory parameter, without which 
weps2sval2.pl will give the following error message.

No input directory specified or mandatory parameter --input not specified. 
Try weps2sval2.pl --help for help.

If the name of the input directory is test then it should have 
"truth_files/official_annotation" folder containing the *.clus.xml in the input
directory. This is required due to the directory structure of the WePS data. 
The program will work without any problems for any other directory names like 
"testdata", "train", etc. 

If the specified input directory does not exists then weps2sval2.pl will give 
the following error.

Can not open input directory <WRONG_DIR_NAME>. Aborting...

=head2 --output 

This option takes as its parameter the directory in which the program 
weps2sval2.pl should store the resulting XML files. This is a mandatory 
parameter and without which weps2sval2.pl will give the following error.

Option output requires an argument
No output directory specified or mandatory parameter --output not specified. 
Try weps2sval2.pl --help for help.

If the specified directory already exists then program halts by giving the 
following error. 

Output directory <OUTPUT_DIR_NAME> already exists, can not proceed.
Try weps2sval2.pl --help for help.

This is done in order to prevent weps2sval2.pl from over writing any existing 
data.

Example: command with the above two options looks as below.

weps2sval2.pl --input testdata/ --output A1out/

The resulting XML is as below.

 <?xml version="1.0" encoding="iso88591" ?>
 <corpus lang='english'>
 <lexelt="Bryan_Adams">
 <instance id="Bryan_Adams_001" docsrc="../testdata//description_files/Bryan_Adams.xml">
 <answer instance="Bryan_Adams_001" senseid="Bryan_Adams_1"/>
 <context>
 TITLE Bryan Adams SNIPPET Official site for Bryan Adams with recent news, tour dates, a song archive, video clips, and a biography. Also find information on new CDs,  DVDs, books, and memorabilia. URL www.bryanadams.com/
 </context>
 </instance>
 .
 .
 .
 </lexelt>
 </corpus>

=head2 --include_discards 

This switch tells weps2sval2.pl to include the instances of 
ambiguous name that were marked as discards by the human annotators of the 
data in the output XML. By default such discards are not included in the output
SenseEval2 format XML.

We have observed during our development and testing that some instances of the 
data do not have entity tags associated with their ranks in the respective 
*.clust.xml. We treat such instances as discards. The name and their ranks for 
which no entities found in the *.clus.xml are as follows.

 Name               Rank	Folder
 Stephen_Clark      48		test
 Stephen_Clark      58		test
 Christine_Borgman  7		traininig
 Christine_Borgman  29		traininig
 Christine_Borgman  61 		traininig

More on the elements with missing entries in XML can be found in the BUGS 
section of the documentation.

Example: command with the --include_discard switch along with the mandatory 
parameter looks as below.

weps2sval2.pl --input testdata/ --output A2out/ --include_discards

The resulting XML is as below.

 <?xml version="1.0" encoding="iso88591" ?>
 <corpus lang='english'>
 <lexelt="Bryan_Adams">
 <instance id="Bryan_Adams_001" docsrc="../testdata//description_files/Bryan_Adams.xml">
 <answer instance="Bryan_Adams_001" senseid="Bryan_Adams_1"/>
 <context>
 TITLE Bryan Adams SNIPPET Official site for Bryan Adams with recent news, tour dates, a song archive, video clips, and a biography. Also find information on new CDs,  DVDs, books, and memorabilia. URL www.bryanadams.com/
 </context>
 </instance>
 .
 .
 .
 <instance id="Bryan_Adams_020" docsrc="../testdata//description_files/Bryan_Adams.xml">
 <answer instance="Bryan_Adams_020" senseid="Bryan_Adams_discard"/>
 <context>
 TITLE Bryan Adams Blog SNIPPET Personal blog for an MIT graduate student interested in robots. He is not the singer, Bryan Adams. URL people.csail.mit.edu/bpadams/blog
 </context>
 </instance>
 </lexelt>
 </corpus>

=head2 --verbose 

This switch tells weps2sval2.pl to display its progress while 
processing the data from the input folder. This switch is useful while 
processing large data sets when the program becomes unresponsive. When 
the switch is specified the command looks as below.

weps2sval2.pl --input testdata/ --output A3out --verbose

and it would generate output like below on the STDOUT.

now processing Violet_Howard.xml:       ************************

now processing Chris_Brockett.xml:      ****************

=head2 --raw 

This switch tells eps2sval2.pl to use the index.html files found in the 
sub folders web_pages/ folder in the directory structure for extracting the 
text. This switch can be used in conjunction with any of the above switches 
to obtain the mixture of results as in the following command examples.

Example: Command with the --raw and --include_discards switches used 
in conjunction. This will result in raw text from the HTML files to be used 
for the creation of the contexts. It will also include HTML files for the 
discarded instances.

weps2sval2.pl --input testdata/ --output A2out/ --raw --include_discards

The resulting XML is as below.

 <?xml version="1.0" encoding="iso88591" ?>
 <corpus lang='english'>
 <lexelt="Bryan_Adams">
 .
 .
 .
 <instance id="Bryan_Adams_016" docsrc="../testdata//description_files/Bryan_Adams.xml">
 <answer instance="Bryan_Adams_016" senseid="Bryan_Adams_1"/>
 <context>
  IMAGE BryanAdams Net IMAGE Home FreeMail FreeFun Mailinglist Download About IMAGE IMAGE IMAGE IMAGE IMAGE IMAGE IMAGE IMAGE IMAGE IMAGE IMAGE IMAGE Hello there Welcome  to BryanAdams Net How are you doing on a day like today I d like to thank the Our visitors fans who visited this site the past three years It s been great to offer  services to such great people This site is temporary unavailable but will be back as soon as possible Bye Remi remi bryanadams net IMAGE IMAGE IMAGE IMAGE IMAGE Copyright  C 1996 2006 RemiStats 
 </context>
 .
 .
 .
 </lexelt>
 </corpus>
 </instance>

Example: Command with the --raw switch used alone. This will result in raw 
text from the HTML files to be used for the creation of the contexts.

weps2sval2.pl --input testdata/ --output A2out/ --raw 

The resulting XML is as below.
 <?xml version="1.0" encoding="iso88591" ?>
 <corpus lang='english'>
 <lexelt="Bryan_Adams">
 .
 .
 .
 <instance id="Bryan_Adams_016" docsrc="../testdata//description_files/Bryan_Adams.xml">
 <answer instance="Bryan_Adams_016" senseid="Bryan_Adams_1"/>
 <context>
  IMAGE BryanAdams Net IMAGE Home FreeMail FreeFun Mailinglist Download About IMAGE IMAGE IMAGE IMAGE IMAGE IMAGE IMAGE IMAGE IMAGE IMAGE IMAGE IMAGE Hello there Welcome  to BryanAdams Net How are you doing on a day like today I d like to thank the Our visitors fans who visited this site the past three years It s been great to offer  services to such great people This site is temporary unavailable but will be back as soon as possible Bye Remi remi bryanadams net IMAGE IMAGE IMAGE IMAGE IMAGE Copyright  C 1996 2006 RemiStats 
 </context>
 .
 .
 .
 </lexelt>
 </corpus>
 </instance>

Example: Command with the --raw, --verbose and --include_discards switches 
used in conjunction. This will work exactly the same as in example with 
--raw and --include_discards options, expect that the --verbose switch will 
make the program display its progress while processing.

weps2sval2.pl --input testdata/ --output A2out/ --raw --include_discards --verbose

=head1 BUGS

WePS data has the following discrepancies.

1. Some contexts have answers in the *.clust.xml but no <doc> </doc> tag 
in corresponding *.xml file in description_files/ folder. They are.

 Name         	Rank		Folder
 Stephen_Clark	33		test
 Violet_Howard	10		test
 Violet_Howard 	18		test

2. Some have <doc> </doc> tag but no answers, as of now we assign 
"discarded" for such unavailable answers to the respective instances. 
Instances that do not have answers are.
 Name               Rank		Folder
 Stephen_Clark      48		test
 Stephen_Clark      58		test
 Christine_Borgman  7		traininig
 Christine_Borgman  29		traininig
 Christine_Borgman  61		traininig

due to such anomalies in the data the expected number of instances 
and answers vary for each folder. Below are the counts that we see 
in the weps2sval2.pl output XML files.

 1. Count of answers.

 for --include_discards switch we should expect the following figures 
 as count of answers.

 traininig 	- 3611.
 test		- 3327.

 for without --include_discards switch we should expect the following 
 figures as count of answers.

 traininig 	- 2340.
 test		- 2875.

 2. Number of instances.

 for --include_discards switch we should expect the following figures 
 as count of instances.

 traininig 	- 3477.
 test		- 2968.

 for without --include_discards switch we should expect the following 
 figures as count of instances.

 traininig 	- 2206.
 test		- 2516.

=head1 AUTHORS
	
Atul Kulkarni, kulka053 at d.umn.edu, University of Minnesota, Duluth

Ted Pedersen, tpederse at d.umn.edu, University of Minnesota, Duluth

Last modified by : 

$Id: weps2sval2.pl,v 1.20 2008/05/23 13:07:03 tpederse Exp $

=head1 COPYRIGHT

Copyright (C) 2007-2008, Atul Kulkarni and Ted Pedersen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, 
USA.

=cut

use File::Find;
use File::Path;
require HTML::TreeBuilder;
require HTML::FormatText;
use Getopt::Long;

if(@ARGV < 1)
{

print STDERR "Usage: weps2sval2.pl --input INPUT-DIRECTORY --output OUTPUT-DIRECTORY\n"; 
print STDERR "Type weps2sval2.pl --help for help.\n";
exit(0);

}

#print the help content here.
sub printHelp()
{
print "\nweps2sval2.pl --input INPUT-DIRECTORY --output OUTPUT-DIRECTORY [--include_discards] [--raw] [--version] [--help]\n";
print "\nThis program accepts the data direcotry name containing the WePS data as input 
and writes its processed output (SensEval-2) formatted XML files in the user 
specified output directory. It by default excludes the documents discarded by  
human annotation. This behaviour can be modified by \"--include_discards\" 
switch. It by default works on the snippet data provided in the description 
xmls. This behaviour can be changed by specifying the \"--raw\" option to 
include the raw html text as well.\n\n";

print "REQUIRED ARGUMENTS:\n\n";

print "--input INPUT-DIRECTORY\t\tThis option specifies which folder 
\t\t\t\tin the above directory to work on.  
\t\t\t\tINPUT-DIRECTORY should be replaced by  
\t\t\t\tvalid directory name that contains 
\t\t\t\tthe WePS data.\n\n";

print "--output OUTPUT-DIRECTORY\tThis is the directory where one 
\t\t\t\tSenseEval-2 formatted file per ambiguous name 
\t\t\t\twill be created. OUTPUT-DIRECTORY should be 
\t\t\t\treplaced by a name of directory that does not 
\t\t\t\texists.\n\n";

print "OPTIONS:\n\n";

print"--include_discards \t\tThis switch tells the program to 
\t\t\t\tinclude the instances already discarded
\t\t\t\tby manual annotation in the WePS 
\t\t\t\tdata. Default is to ignore such instances.\n\n";

print"--raw \t\t\t\tThis switch is used to indicate to 
\t\t\t\tthe program if he user intends 
\t\t\t\tto use the raw text 
\t\t\t\tfrom the html or the snippet data 
\t\t\t\tprovided with the xmls in the 
\t\t\t\tweb_pages folder structure.\n\n";

print"--verbose \t\t\tThis switch tells the program to display its
\t\t\t\tprogress to the user. This avoids making it  
\t\t\t\tlook unresponsive while processing big chunk 
\t\t\t\tof data. Program prints the name of the file 
\t\t\t\t it is currently processing along with the 
\t\t\t\tcurrent progress with '*', as it completes 
\t\t\t\tthe processing. By default does not print 
\t\t\t\tanything to the STDOUT.\n\n";

print"--version \t\t\tDisplays version information.\n\n";

print "--help \t\t\t\tDisplays help for this command.\n\n";

}


#declare command line option variables.
my $input = undef; #Input directory in the home directory of WePS.
my $output = undef; #The directory where user expects the output files.
my $inc_discard = "false"; #tells the program if it should include discards or not.
my $verbose = 0;

GetOptions("input=s" 		=> \$input,
	   "output=s" 		=> \$output,
	   "include_discards"	=> \$inc_discard,
	   "raw" 		=> \$raw,
	   "verbose"		=> \$verbose,
   	   "version" 		=> \$version,
	   "help" 		=> \$help);

#Reads for --version displays information.
if($version == 1){
	$string ='$Id: weps2sval2.pl,v 1.20 2008/05/23 13:07:03 tpederse Exp $';
	print "$string\n";
	exit(0);
}

#Reads for --help and displays help information.
if($help == 1){
	printHelp();
	exit(0);
}

#Reads for --input option and its argument.
if (defined($input)){
	if(!(-r $input)){
		print STDERR "Can not open input directory "."$input".". Aborting...\n";
		exit(0);
	}
}else{
	print STDERR "No input directory specified or mandatory parameter --input not specified. \nTry weps2sval2.pl --help for help.\n";
	exit(0);
}

#Reads for --output and its argument.
if(defined($output)){
	if( -e $output ){	#print error if it already exists.
		print STDERR "Output directory $output already exists, can not proceed.\nTry weps2sval2.pl --help for help.\n";
		exit(0);
	}
	else{	#i does not exists then create it for our use.
		if(!(mkdir "$output", 0744)){
			print STDERR "Can not create output directory $output because $!. \n";
			exit(0);
		}
	}					
}else{
	print STDERR "No output directory specified or mandatory parameter --output not specified. \nTry weps2sval2.pl --help for help.\n";
	exit(0);	
}

#Reads for --include_discards and sets falg to "true".
if($inc_discard == 1){
	$inc_discard = "true";
}



#Misc. utility functions.
my @XMLs;
sub list_files{
	if(m/^.*\.(xml)$/i){
		push(@XMLs, "$File::Find::name\n");
        }
}

find(\&list_files, "$input/description_files/");
@XMLs = sort(@XMLs);
#Program Logic.

#Read one XML at a time and Just create the SensEval-2 format out put in output directory.
foreach $XML (@XMLs){

	chomp ($XML);
	%XMLhash = ();
	$_ = $XML;
	m/^(.*)\/(.*)\.xml$/;
	$name = $2;
	
	#If --verbose switch is specified then just print the name of processing XML.
	if($verbose == 1){
		printf("\nnow processing %20s.xml : ",$name);
		$| = 1;
	}

	#Parse the XML we might get Title and Snippet, but at times it is just not present.
	open (DESCXML, $XML) or die "can not open file $XML as $!\n";
	while(<DESCXML>){
		#we might or might not get title so this case handles when we have it
		if (/<doc rank=\"(\d+)\" title=\"(.*)\" url=\"(.+)\">/) { 
			$rank = $1;
			$title = $2;
			$url = $3;
		}

		#Handles the case when we do not have title information.		
		if (/<doc rank=\"(\d+)\" url=\"(.+)\" \/>/) { 
			$rank = $1;
			$url = $2;
			if(!(defined $XMLhash{$rank})){
				$XMLhash{$rank} = "$rank@#_@#_@#$url";
			}else{
				print STDERR "Duplicate ranks. $rank\n";
			}
		}
		
		#Handles case where snippet information is available.	
		if (/<snippet>(.+)<\/snippet>/) {
			$snippet = $1;
		}
		
		#at the end of the record put it in hash.
		if(/<\/doc\>/){
			if(!(defined $XMLhash{$rank})){
				$XMLhash{$rank} = "$rank@#$title@#$snippet@#$url";
			}else{
				print STDERR "Duplicate ranks $rank found for $name. \n";
			}
		}
	}
	close(DESCXML);

	#now we need to read the corresponding XML containing answes and parse it.
	#this XML tells us per rank information about clusters in which the data is 
	#classified under manual classification.
	$_ = $input;
	if (m/^(.+)\/test(\/)?$/){
		$path = "$input/truth_files/official_annotation";
	}else{
		$path = "$input/truth_files";
	}
	$CXML = "$path/$name.clust.xml";
	%CLUSThash = ();
	#open respective clust.xml and read it in o a hash.
	open (CXML, $CXML) or die "Can not open $CXML as $!";
	while(<CXML>){
		#Entity Id can be a Number or discarded.
		if (/<entity id=\"(\S+)\"/) {
			$entity = $1;
		}

		if (/<discarded/) {
			$entity = 'discard';
		}		
		
		#Push entity for the ranks.
		#Some ranks might have multiple entities.
		
		if (/<doc rank=\"(\S+)\"/) {
			$crank = $1;
			if(defined $CLUSThash{$crank}){
				$CLUSThash{$crank} = $CLUSThash{$crank}." ".$entity;
			}else{
				$CLUSThash{$crank} = "$entity";
			}
		}
	}
	close(CXML);
	#cluster XML read and hash created. 

	#Initiate the XML.
	$doc = "";
	$doc = "<\?xml version=\"1.0\" encoding=\"iso88591\" ?>\n";
	$doc = $doc . "<corpus lang=\'english\'>\n";
	$doc = $doc . "<lexelt=\"$name\">\n";
	
	#Gather all the ranks for the description xml
	#for each rank we will find answer and embed
	#in context. Such contexts are appended to the
	#xml document. Once complete write Xml out.
	@ranks = sort {$a <=> $b} keys %XMLhash;
	
	#we calculate the ranks per star for displaying output.
	my $total_ranked_documents = $#ranks;
	my $docs_per_star = int($total_ranked_documents / 20);
	if($docs_per_star < 0)
	{
		$docs_per_star = 1;
	}
	my $doc_count = 0;

	foreach $key (@ranks){
		#Get the data for Snippet, Title, URL.
		if($verbose == 1){
			$doc_count++;
			if($doc_count > $docs_per_star){
				print "*";
				$| = 1;
				$doc_count = 1;
			}
		}			
		
		$desc_record = $XMLhash{$key};

		#Some ranks do not have answers e.g. Violet_Howard and Stephen_Clark.
		#we assign NO_ANSWER to them as the dummy answer. 
		if(defined $CLUSThash{$key}){
			$ans_reacord = $CLUSThash{$key};
		}else{
			$ans_reacord = "discard";
		}
		
		#Split the records from the string.
		@desc_rec = split("@#", $desc_record);
		@answers = split(" ", $ans_reacord);

		#start Instance.
		$instance = sprintf ("<instance id=\"%s_doc_%03d\" docsrc=\"%s\"\>\n", $name,$desc_rec[0],$XML);

		#To this instance we append the answers in the format. 
		#<answer instance="FirstName_LastName_Rank" senseid="FirstName_LastName_ID"/>
		#we extract answers for this rank ($key) and then create such tags.
		#Answers are useful for evaluating clusters with Sense Clusters.
		#create answer tags for rank.
		foreach $answer (@answers){
			#By default we exclude the discarded contexts. 
			#Hence if an answer is discard and we do not have 
			#--include_discards switch enabled. then ignore such a context.
			if($answer eq "discard" && $inc_discard eq "false"){
				$discard = 1;
				last;
			}else{
				$instance = $instance . sprintf ("<answer instance=\"%s_entity_%03d\" senseid=\"%s_%s\"\/>\n",	$name, $desc_rec[0], $name,
 $answer);		}
 		}

		#To continue to next key in the array we need to use this condition.
		#It just checks if $discard is set from the loop above, if so it 
		#continue to next key after reseting $discard.
		if($discard == 1){
			$discard =0;
			next;
		}
		#Check if --raw specified. If so just make the context from the HTMLs and not the snippet text. 
		#But if not specified then use Snippet text for this.
		if($raw == 0){		
			#Create context out of the Snippet and Title Data when available 
			#and rest just create the Instance of URL.
			if($desc_rec[1] eq "_" && $desc_rec[2] eq "_"){
				$instance = $instance . "<context>\n" . "URL $desc_rec[3]\n" . "</context>\n" . "</instance>\n";

			}else{
				$instance = $instance . "<context>\n" . "TITLE $desc_rec[1] SNIPPET $desc_rec[2] URL $desc_rec[3]\n" . "</context>\n" . "</instance>\n";
			}
		}else{
			$rnk = "";
			if($key < 10){
				$rnk = sprintf("00%1d",$key);
			}else{ 
				if($key < 100){
					$rnk = sprintf("0%d",$key);				
				}else{
					$rnk = $key;			
				}
			}
			$web_folder = "$input/web_pages/$name/raw/$rnk";
			if (!(-e $web_folder)){
				next;
			}
			#Code taken form format-raw.pl - Start.
			$tree = HTML::TreeBuilder->new->parse_file("$web_folder/index.html");
			$formatter = HTML::FormatText->new(leftmargin => 0, rightmargin => 50);

			# $_ contains the formatted html page, with line feeds still intact
			$_ = $formatter->format($tree);

			# remove all non alphanumerics, which will eliminate line feeds too
			s/[^A-Za-z0-9 ]/ /g;

			# remove extra white spaces
			s/\s\s+/ /g;

			if (/^\s+$/) {
				$instance = $instance . "<context>\n" . "BLANKCONTEXT\n" . "</context>\n" . "</instance>\n";
			}else{ 
				 $instance = $instance . "<context>\n" . "$_\n" . "</context>\n" . "</instance>\n";
			}

			#Code taken form format-raw.pl - end

		}
		$doc = $doc . "$instance";	
	}#end for loop for extracting the context information. 

	$doc = $doc . "</lexelt>\n" . "</corpus>\n";
	open (OUTXML, ">$output/$name.xml");
	print OUTXML "$doc";
	close OUTXML; 
}#End of for loop for XML from description files.

#This just pushes the new line character at the end of the last star.
if($verbose == 1){
	print "\n";
}

